#include <stdlib.h>
#include <stdio.h>

#include "array.h"

int* create_tab(int tab[], unsigned tab_size) {
    tab = malloc(sizeof(int) * tab_size);
    if (tab != NULL) {
        /* initialize to zero the integer array */
        for (unsigned i = 0; i < tab_size; i++) {
            tab[i] = 0;
        }
    }
    return tab;
}

void free_tab(int tab[]) {
    if (!tab)
        free(tab);
}

/* we suppose both tabs are already created */
static void copy_tab(int src_tab[], int dest_tab[], unsigned src_tab_size, unsigned index_offset) {
    /* FIXME: I think it's worth doing some sanity checks on the array size:
     * dest_tab_size >= src_tab_size */
    if (src_tab == NULL || dest_tab == NULL) {
        printf("Please ensure you have created both arrays beforehand\n");
        return;
    }
    for (unsigned i = 0; i < src_tab_size; i++) {
        dest_tab[i + index_offset] = src_tab[i];
    }
}

/* one must free the two source tabs in case they will be unused after to concatenation  */
int* concat_tab(int tab1[], unsigned tab_size1, int tab2[], unsigned tab_size2) {
    int* tab_dest = NULL;
    tab_dest = create_tab(tab_dest, tab_size1 + tab_size2);

    copy_tab(tab1, tab_dest, tab_size1, 0);
    copy_tab(tab2, tab_dest, tab_size2, tab_size1);
    return tab_dest;
}

int* resize_tab(int tab[], unsigned old_tab_size, unsigned new_tab_size) {
    tab = realloc(tab, sizeof(int) * new_tab_size);
    /* zero by default the added cells */
    if (old_tab_size < new_tab_size) {
        for (unsigned i = old_tab_size; i < new_tab_size; i++) {
            tab[i] = 0;
        }
    }
    return tab;
}

/* number of occurences of an element in an unsorted array  */
unsigned count_tab_element(int tab[], unsigned tab_size, int element) {
    unsigned el_count = 0;

    for (unsigned i = 0; i < tab_size; i++) {
        if (tab[i] == element) {
            el_count++;
        }
    }
    return el_count;
}

unsigned count_tab_criteria(int tab[], unsigned tab_size, c_criteria_cb c_criteria) {
    unsigned cr_count = 0;

    for (unsigned i = 0; i < tab_size; i++) {
        if (c_criteria(tab[i])) {
            cr_count++;
        }
    }
    return cr_count;
}

bool is_even(int a) {
    return (a % 2 == 0);
}

bool is_odd(int a) {
    return (a % 2 != 0);

}

void sort_tab(int tab[], unsigned tab_size, s_criteria_cb sort_criteria) {
    sort_bubble_array(tab, tab_size, sort_criteria);
}
